/*-----------------------------------------------------------------------

                         SYRTHES version 4.1
                         -------------------

     This file is part of the SYRTHES Kernel, element of the
     thermal code SYRTHES.

     Copyright (C) 2009 EDF S.A., France

     contact: syrthes-support@edf.fr


     The SYRTHES Kernel is free software; you can redistribute it
     and/or modify it under the terms of the GNU General Public License
     as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.

     The SYRTHES Kernel is distributed in the hope that it will be
     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.


     You should have received a copy of the GNU General Public License
     along with the SYRTHES Kernel; if not, write to the
     Free Software Foundation, Inc.,
     51 Franklin St, Fifth Floor,
     Boston, MA  02110-1301  USA

-----------------------------------------------------------------------*/

# include <stdio.h>
# include <stdlib.h>
# include <math.h>

#include "syr_usertype.h"
#include "syr_tree.h"
#include "syr_bd.h"
#include "syr_parall.h"
#include "syr_abs.h"
#include "syr_option.h"
#include "syr_const.h"
#include "syr_proto.h"

#ifdef _SYRTHES_MPI_
#include "mpi.h"
MPI_Status status;
#endif

extern int user_stop;     /* pour generer un arret prmature mais propre du code */ 
extern char nommeteo[CHLONG];
extern int nbVar;
extern FILE *fadd;

/* Dans les fonctions qui suivent, on pourra acceder aux grandeurs suivantes :
/    - maillnodes.coords[j][i] : coordonnee j du noeud i
/    - maillnodes.nodes[j][i]  : noeud j de l'element i
/    - t[i]                    : temperature au noeud i
/    - maillnodes.nrefe[i]     : reference de l'element i
/
/    - maillnodes.ndim  : dimension du probleme
/    - maillnodes.nelem : nombre d'elements du maillage
/    - maillnodes.npoin : nombre de noeud du maillage
/
/ fonction data_element_moy permet de recuperer les donnees moyennes sur l'element 
/ -------------------------
/   --> data_element_moy(i,maillnodes,var,&nrefe,&x,&y,&z,&v);
/   avec en entree :
/            i =numero de l'element 
/          var =champ sur les noeuds (en general temperature)
/   et en retour :
/        nrefe = numero de materiau de l'element,              
/    x,y,z = coordonnees barycentriques de l'element   
/        v = valeur moyenne de la variable sur l'element
/
/ fonction interpol_table1D permet d'interpoler dans une table de dimension 1
/ -------------------------
/ (TABLE (tabX) = tabFX)
/   --> y = interpol_table1D(double x,int nb,double *tabX,double *tabFX)
/   avec en entree :
/            nb = la longueur de la table
/          tabX = la liste des entrees de la table (tabX[nb])
/         tabFX = la liste des valeurs de la variable (tabFX[nb])
/   et en retour :
/         y = valeur interpolee
/
/
/ cas des calculs paralleles
/ --------------------------
/ les fonctions suivantes peuvent etre appelees pour realiser des operations
/ sur l'ensemble des processeurs
/ s=somme_int_parall(n)  ou s=somme_double_parall(x) : sommer une variable
/ mx=max_int_parall(n)   ou mx=max_double_parall(x)   : calculer le max d'une variable
/ mn=min_int_parall(n)   ou mn=min_double_parall(x0   : calculer le min d'une variable
/
/ Rq : ces fontions fonctionnent egalement si le calcul est lance en sequentiel
/
/ Pour n'avoir les impressions qu'a 1 seul exemplaire dans le fichier listing,
/ utiliser le test suivant :
/ if (syrglob_nparts==1 ||syrglob_rang==0)
/    printf("mon impression\n");
*/



/*|======================================================================|
  | SYRTHES 4.1                                       COPYRIGHT EDF 2009 |
  |======================================================================|
  | AUTEURS  : I. RUPP, C. PENIGUEL                                      |
  |======================================================================|
  |        Proprietes physiques                                          |
  |======================================================================| */
void user_cini(struct Maillage maillnodes,double *t,struct PasDeTemps *pasdetemps,
         struct Meteo meteo,struct Myfile myfile)
{
  int i;
  double x,y,z;

  /* si on souhaite programmer ses propres formules,
  /   mettre la variable mesproprietes a 1,
  /   puis fournir les formules */

  int mescondinit=0;

  if (mescondinit)
    {
    }
}


/*|======================================================================|
  | SYRTHES 4.1                                       COPYRIGHT EDF 2009 |
  |======================================================================|
  | AUTEURS  : I. RUPP, C. PENIGUEL                                      |
  |======================================================================|
  |        Proprietes physiques                                          |
  |======================================================================| */
void user_cphyso(struct Maillage maillnodes,
     double *t,struct Prophy physol,struct PasDeTemps *pasdetemps,
     struct Meteo meteo,struct Myfile myfile)
{
  int i,j,nrefe,ng,nr;
  double x,y,z,tmoy,ct;
  static int prem;

  /* si on souhaite programmer ses propres formules,
  /   mettre la variable mesproprietes a 1,
  /   puis fournir les formules */

  int mesproprietes=0;

  if (mesproprietes)
    {
    }
}
/*|======================================================================|
  | SYRTHES 4.1                                       COPYRIGHT EDF 2009 |
  |======================================================================|
  | AUTEURS  : I. RUPP, C. PENIGUEL                                      |
  |======================================================================|
  |        Conditions aux limites                                        |
  |======================================================================| */
void user_limfso(struct Maillage maillnodes,struct MaillageBord maillnodebord,  
     struct MaillageCL maillnodeus,
     double *t,struct Clim diric,struct Clim flux,
     struct Clim echang,struct Clim rayinf,struct PasDeTemps *pasdetemps,
     struct Meteo meteo,struct Myfile myfile)
{
  int i,j,nr,np;
  double x,y,z,xt,ct;
  
  /* si on souhaite programmer ses propres formules,
  /   mettre la(les) variables ci dessous a 1,
  /   puis fournir les formules */
  int mesflux=0;
  int mesdirichlet=0;
  int mescoeffech=0;
  int mesrayinf=0;

  /* current time (s) */
  ct=pasdetemps->tempss;

  /* Conditions de flux */
  /* ------------------ */
  if (mesflux)
    {
    }

  /* Conditions d'echange */
  /* -------------------- */
  if (mescoeffech)
    {
    }
 
  /* Conditions de rayonnement infini */
  /* -------------------------------- */
  if (mesrayinf)
    {
    }
}
      
/*|======================================================================|
  | SYRTHES 4.1                                       COPYRIGHT EDF 2009 |
  |======================================================================|
  | AUTEURS  : I. RUPP, C. PENIGUEL                                      |
  |======================================================================|
  |        Flux volumiques sur la temperature                            |
  |======================================================================| */
void user_cfluvs(struct Maillage maillnodes,
     double *t,struct Cvol fluxvol,struct PasDeTemps *pasdetemps,
     struct Meteo meteo,struct Myfile myfile)
{
  int i,j,k,ng,nr,nrefe;
  double x,y,z,tmoy,ct;
  double g1,g2,z1,z2;
  
  /* si on souhaite programmer ses propres formules,
  /   mettre la variable ci dessous a 1,
  /   puis fournir les formules */
  int mesfluxvol=1;
  
  /* Co60 uniquement pour le moment : a modifier */
  int ref_colis = 1; /* référence du volume des colis où appliquer le flux volumique */
  double Pth_N1 = 2.43; /* Puissance thermique (en W/m3) de la nappe 1 */
  double Pth_N2 = 4.15; /* Puissance thermique (en W/m3) de la nappe 2 */
  double Pth_N3 = 10.05; /* Puissance thermique (en W/m3) de la nappe 3 */
  double Pth_N4 = 31.18; /* Puissance thermique (en W/m3) de la nappe 4 */
  double hcolis = 1.3; /*hauteur d'un colis en metre */
  
  /* Conditions de flux volumiques */
  /* ----------------------------- */
  if (mesfluxvol==1)
  {   
    for (i=0;i<fluxvol.nelem;i++) 
    {
      ng=fluxvol.nume[i];          /* numero global de l'element       */
      data_element_moy(ng,maillnodes,t,&nrefe,&x,&y,&z,&tmoy);
	
      if (nrefe==ref_colis)
      {
        if (z < hcolis)		
		  fluxvol.val2[i]=Pth_N1;
	    else if (z <2.0*hcolis)
		  fluxvol.val2[i]=Pth_N2;
	    else if (z <3.0*hcolis)
		  fluxvol.val2[i]=Pth_N3;
	    else
		  fluxvol.val2[i]=Pth_N4;
      }
	}
  }
}

/*|======================================================================|
  | SYRTHES 4.1                                       COPYRIGHT EDF 2009 |
  |======================================================================|
  | AUTEURS  : I. RUPP, C. PENIGUEL                                      |
  |======================================================================|
  |        Conditions aux limites - resistances de contact               |
  |======================================================================| */
void user_rescon(struct Maillage maillnodes,struct MaillageCL maillnodeus,
         double *t,double *tcor,struct Contact rescon,struct PasDeTemps *pasdetemps,
         struct SDparall sdparall)
{
  int i,j,nrefe,ne,num;
  double t1,t2,ct;

  /* si on souhaite programmer ses propres formules,
  /   mettre la variable ci dessous a 1, puis fournir les formules */
  int mesrescon=0;

  if (mesrescon==0)
    return;
  else
    {
    }
  
}

